#include <string.h>
#include "eth_prop.h"
#include "eth.h"
#include "eth_drv.h"
#include "misc.h"
#include "itmr.h"
#include "error.h"
#include "main.h"
#include "uart.h"

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//   IP 
#define ip_Equ(ip1, ip2)	\
		(((ip1).d8[0] == (ip2).d8[0]) && \
		 ((ip1).d8[1] == (ip2).d8[1]) && \
		 ((ip1).d8[2] == (ip2).d8[2]) && \
		 ((ip1).d8[3] == (ip2).d8[3]))

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//   ARP 
typedef struct{
	TEthAddr mac_addr;
	TIP4Addr ip4_addr;
	unsigned int reg_time;			//    		
	//   - 
} TARPTableElem, *pTARPTableElem;

#define ARP_TABLE_SIZE	16	//    ARP 
#define ARP_REC_TIME_LIFE	180000	//     ARP  - 3 

// ARP 
static TARPTableElem ARPTable[ARP_TABLE_SIZE];
#define PING_INT	60000
#define UDP_INT		60000

static TEthAddr EthernetMAC;
static TIP4Addr EthernetIP;
static TIP4Addr SubNetMask;
static TIP4Addr Gateway;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//  () 
#define SND_CELL_SIZE	1500

static unsigned char TempSnd[SND_CELL_SIZE];
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//  
#define RCV_BUF_SIZE	1500
unsigned char RcvBuf[RCV_BUF_SIZE];

//------------------------------------------------------------------------------																				
//      
static bool CheckSubNetMask(TIP4Addr src_ip, TIP4Addr dst_ip, TIP4Addr mask)
{
	if(((src_ip.d8[0] & mask.d8[0]) == (dst_ip.d8[0] & mask.d8[0])) && 
	   ((src_ip.d8[1] & mask.d8[1]) == (dst_ip.d8[1] & mask.d8[1])) &&
	   ((src_ip.d8[2] & mask.d8[2]) == (dst_ip.d8[2] & mask.d8[2])) && 
	   ((src_ip.d8[3] & mask.d8[3]) == (dst_ip.d8[3] & mask.d8[3])))
	   return true;
	
	return false;
}

//-------------------------------------------------------
//-------------------------------------------------------
//  
static TEthernetHeader ethernet_MakeHeader(TEthAddr dst, TEthAddr src, unsigned short type)
{
	TEthernetHeader hdr;
	
	hdr.dstMAC = dst;
	hdr.srcMAC = src;
	hdr.type = type;
	
	return hdr;	
}

//---------------------------------------------------------------------------------------------
//    ARP 
static unsigned int arp_FindOldARPRec(void)
{	
	unsigned int ind, old_reg_time, old_reg_ind;
	old_reg_time = ARPTable[0].reg_time;
	old_reg_ind = 0;
	for(ind = 1; ind < ARP_TABLE_SIZE; ind++){
		if(ARPTable[ind].reg_time < old_reg_time){
			old_reg_time = ARPTable[ind].reg_time;		
			old_reg_ind = ind;
		}
	}

	return old_reg_ind;
}

//---------------------------------------------------------------------------------------------
//  ARP 
static void arp_UpdataARPTable(TEthAddr mac, TIP4Addr ip4)
{
	unsigned int ind, new_rec_ind;	

	for(ind = 0; ind < ARP_TABLE_SIZE; ind++){
		//     ,     
		if(!memcmp(&(ARPTable[ind].mac_addr), &mac, sizeof(TEthAddr))
		&& !memcmp(&(ARPTable[ind].ip4_addr), &ip4, sizeof(TIP4Addr))){
			ARPTable[ind].reg_time = itmr_GetTick();						
			return;
		}		
	}
	//         	
	
	//   ARP 	
	new_rec_ind = arp_FindOldARPRec();
	
	ARPTable[new_rec_ind].mac_addr = mac;
	ARPTable[new_rec_ind].ip4_addr = ip4;	
	ARPTable[new_rec_ind].reg_time = itmr_GetTick();		
}

//---------------------------------------------------------------------------------------------
//   ARP 
void arp_TimeOutARPRec(void)
{	
	unsigned int ind;
	
	for(ind = 0; ind < ARP_TABLE_SIZE; ind++){
		//   -   	
		if(ARPTable[ind].reg_time)
			//     - 
			if(itmr_IsTime(ARPTable[ind].reg_time, ARP_REC_TIME_LIFE)){
				memset(&ARPTable[ind], 0, sizeof(TARPTableElem));	
			}												
	}					
}

//------------------------------------------------------------------------------
//  ARP 
//     0 -    
static unsigned short arp_Parser(void)
{		
	ethprop_GetIP(&EthernetIP);
	//  ARP :  Ethernet, IPv4
	// Ethernet	
	if(ARP_HDR(RcvBuf)->hrdw_type == HRDW_TRAN_PROT_ETHERNET){				
		// IPv4		
		if(ARP_HDR(RcvBuf)->prot_type == PROT_CODE_IPv4){								
			//   			
			if(ARP_HDR(RcvBuf)->hrdw_addr_length == HRDW_ADDR_LENGTH_ETHERNET){												
				//   				
				if(ARP_HDR(RcvBuf)->prot_addr_length == LGC_ADDR_LENGTH_IPv4){										
					//    ,     
					//    ARP 										
					arp_UpdataARPTable(ARP_HDR(RcvBuf)->src_hrdw_addr, ARP_HDR(RcvBuf)->src_prot_addr);
					
					//     															
					if(ip_Equ((ARP_HDR(RcvBuf)->dst_prot_addr), EthernetIP)){		
						//   						
						return ARP_HDR(RcvBuf)->op_code;														
					}							
				}				
			}			
		} 
	}				
	
	return 0;		
}

//---------------------------------------------------------------------------------------------
//   ARP
static TARPPack arp_MakeHeader(unsigned short op_code, 
						TEthAddr src_hrdw_addr, TIP4Addr src_prot_addr,
						TEthAddr dst_hrdw_addr, TIP4Addr dst_prot_addr)
{
	TARPPack hdr;
	
	//  
	hdr.hrdw_type = HRDW_TRAN_PROT_ETHERNET;
	hdr.prot_type = PROT_CODE_IPv4;
	hdr.hrdw_addr_length = HRDW_ADDR_LENGTH_ETHERNET;
	hdr.prot_addr_length = LGC_ADDR_LENGTH_IPv4;
	//    
	hdr.op_code = op_code;
	hdr.src_hrdw_addr = src_hrdw_addr;
	hdr.src_prot_addr = src_prot_addr;
	hdr.dst_hrdw_addr = dst_hrdw_addr;
	hdr.dst_prot_addr = dst_prot_addr;			

	return hdr;
}

//---------------------------------------------------------------------------------------------
//  ARP 
static bool arp_Answer(void)
{
	TEthernetHeader	eth_hdr;
	TARPPack arp_pack;

	unsigned int dcnt = ETHERNET_HDR_SIZE + ARP_HEADER_SIZE;		

	//---------------------------------
	//   ARP 
	//   ethernet'
	//   =  ,   	
	ethprop_GetMAC(&EthernetMAC);
	ethprop_GetIP(&EthernetIP);
	eth_hdr = ethernet_MakeHeader(ETH_HDR(RcvBuf)->srcMAC, EthernetMAC, ETHTYPE_ARP);	
	//   arp'	
	arp_pack = arp_MakeHeader(ARP_OP_CODE_ANSWER, 
					//  
					EthernetMAC, EthernetIP,	
					//  
					ETH_HDR(RcvBuf)->srcMAC, ARP_HDR(RcvBuf)->src_prot_addr);
	
	//---------------------------------
	memcpy(TempSnd, &eth_hdr, ETHERNET_HDR_SIZE);
	memcpy(TempSnd + ETHERNET_HDR_SIZE, &arp_pack, ARP_HEADER_SIZE);
	
		
	return enc28j60_send_packet(TempSnd, dcnt);		
}

//---------------------------------------------------------------------------------------------
//  ARP   ip_addr
bool arp_Request(TIP4Addr* ip_addr)
{	
	static const TEthAddr	EthernetBroadcast 	= {{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}};
	static const TEthAddr EthernetZero 		= {{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}}; 
	TEthernetHeader	eth_hdr;
	TARPPack arp_pack;
	unsigned int dcnt = ETHERNET_HDR_SIZE + ARP_HEADER_SIZE;
	TIP4Addr ip_dst = *ip_addr;
	
	ethprop_GetMAC(&EthernetMAC);
	ethprop_GetIP(&EthernetIP);
	ethprop_GetGate(&Gateway);
	
	//   ,   
	if(!CheckSubNetMask(EthernetIP, *ip_addr, SubNetMask))	
		ip_dst = Gateway;					
	
	//  	
	eth_hdr = ethernet_MakeHeader(EthernetBroadcast, EthernetMAC, ETHTYPE_ARP);	
	arp_pack = arp_MakeHeader(ARP_OP_CODE_REQUEST, 
					//  
					EthernetMAC, EthernetIP,	
					//  
					EthernetZero, ip_dst);
	//---------------------------------

	memcpy(TempSnd, &eth_hdr, ETHERNET_HDR_SIZE);
	memcpy(TempSnd + ETHERNET_HDR_SIZE, &arp_pack, ARP_HEADER_SIZE);		
			
	return enc28j60_send_packet(TempSnd, dcnt);
}

//---------------------------------------------------------------------------------------------
//   IP
static TPseudoHdr ip_MakePseudoHdr(TIP4Addr src, TIP4Addr dst, unsigned short prot, unsigned short length)
{
	TPseudoHdr ps_hdr;
	ps_hdr.src_addr = src;
	ps_hdr.dst_addr = dst;		
	ps_hdr.protocol = EXCHANGE(prot);
	ps_hdr.length 	= EXCHANGE(length);

	return ps_hdr;
}

//------------------------------------------------------------------------------																				
//  IP 
static unsigned int ip_Parser(void)
{
	unsigned int res = 0;	
	ethprop_GetIP(&EthernetIP);	
	//   
	if(HINIBBLE(IP_HDR(RcvBuf)->version) == 4){		
		//     ,  5 * 4 = 20 
		if(LONIBBLE(IP_HDR(RcvBuf)->version) == 5){						
			//  ARP 			
			arp_UpdataARPTable(ETH_HDR(RcvBuf)->srcMAC, IP_HDR(RcvBuf)->src_ip);
			//    IP 			
			if(ip_Equ(IP_HDR(RcvBuf)->dst_ip, EthernetIP)){
				res = IP_HDR(RcvBuf)->prot;
				if(res == POH_UDP)					
					ip_MakePseudoHdr(	IP_HDR(RcvBuf)->src_ip, IP_HDR(RcvBuf)->dst_ip,
														IP_HDR(RcvBuf)->prot, IP_HDR(RcvBuf)->length);											
			}										
		}
	}									
		
	return res;				
}

//------------------------------------------------------------------------------																				
//   IP
static TIPHeader ip_MakeHdr(unsigned short length, unsigned short id,
					 unsigned short off, unsigned char ttl, unsigned char prot,
					 TIP4Addr src_ip, TIP4Addr dst_ip)
{
	TIPHeader hdr;
	//   IP   4,   IP   unsigned int
	// ,     5  	
	hdr.version = 0x45; 		
	hdr.service = 0;	 		//  
	hdr.length = EXCHANGE(length);		//  
	hdr.id = id;				// 
	hdr.off = off;				// ,        3  
	hdr.ttl = ttl;				//   
	hdr.prot = prot;			//   .  (TCP, UDP...)
	hdr.cs = 0;					//   
	hdr.src_ip = src_ip;		// IP   		
	hdr.dst_ip = dst_ip;		// IP   

	//  
	hdr.cs = CalcHeaderCS(&hdr, IP_HEADER_SIZE);

	return hdr;	
}

//------------------------------------------------------------------------------																				
//  ping . 
static unsigned char icmp_Parser(void)
{	
	if((ICMP_HDR(RcvBuf)->code == CODE_ECHO_REQUEST) && (ICMP_HDR(RcvBuf)->type == TYPE_ECHO_REQUEST))
		return CODE_REQUEST;
	else{
		if((ICMP_HDR(RcvBuf)->code == CODE_ECHO_REPLY) && (ICMP_HDR(RcvBuf)->type == TYPE_ECHO_REPLY))
			return CODE_REPLY;
		else
			return CODE_ERROR;		
		}
}

//---------------------------------------------------------------------------------------------
//   ICMP
static TICMPHeader icmp_MakeHeader(unsigned char type, unsigned char code)
{
	TICMPHeader hdr;
	
	hdr.type = type;
	hdr.code = code;
	hdr.cs = 0;	//      ICMP 
	
	return hdr;
}


//---------------------------------------------------------------------------------------------
//    ping
static bool icmp_Answer(void)
{
	
	TEthernetHeader eth_hdr;	//   Ethernet
	TIPHeader	ip_hdr;			//  IP
	TICMPHeader icmp_hdr;		//  ICMP
		
	unsigned int full_size = ETHERNET_HDR_SIZE + EXCHANGE(IP_HDR(RcvBuf)->length) + 1;
	unsigned short data_size = EXCHANGE(IP_HDR(RcvBuf)->length) - IP_HEADER_SIZE - ICMP_HEADER_SIZE;
	unsigned int cs = 0;
		
	//  MAC header
   	eth_hdr = ethernet_MakeHeader(ETH_HDR(RcvBuf)->srcMAC, EthernetMAC, ETHTYPE_IPv4);	
	//  IP header
	ip_hdr = ip_MakeHdr(EXCHANGE(IP_HDR(RcvBuf)->length), IP_HDR(RcvBuf)->id, IP_HDR(RcvBuf)->off, IP_HDR(RcvBuf)->ttl, 
													   IP_HDR(RcvBuf)->prot, IP_HDR(RcvBuf)->dst_ip, IP_HDR(RcvBuf)->src_ip);
	//  ICMP header
	icmp_hdr = icmp_MakeHeader(TYPE_ECHO_REPLY, CODE_ECHO_REPLY);
	
	cs += CalcSum(&icmp_hdr, ICMP_HEADER_SIZE);
	cs += CalcSum((unsigned char*)(RcvBuf + ETHERNET_HDR_SIZE + IP_HEADER_SIZE + ICMP_HEADER_SIZE), data_size);
	
	while(cs >> 16)
		cs = (cs & 0xFFFF) + (cs >> 16);

	icmp_hdr.cs = (unsigned short)(~cs);	 
		
	memcpy(TempSnd, &eth_hdr, ETHERNET_HDR_SIZE);
	memcpy(TempSnd + ETHERNET_HDR_SIZE, &ip_hdr, IP_HEADER_SIZE);
	memcpy(TempSnd + ETHERNET_HDR_SIZE + IP_HEADER_SIZE, &icmp_hdr, ICMP_HEADER_SIZE);	
	memcpy(TempSnd + ETHERNET_HDR_SIZE + IP_HEADER_SIZE + ICMP_HEADER_SIZE,
			RcvBuf + ETHERNET_HDR_SIZE + IP_HEADER_SIZE + ICMP_HEADER_SIZE, data_size);	
		
	return enc28j60_send_packet(TempSnd, full_size);	
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//      
void eth_FrameProcessing(void)
{	
		if(readRegByte(EPKTCNT)!= 0){							
			//    
					
			enc28j60PacketReceive(RcvBuf,RCV_BUF_SIZE);
			
			//   
			switch(ETH_HDR(RcvBuf)->type){
				case ETHTYPE_ARP:				
					switch(arp_Parser()){
						
						// 					
						case ARP_OP_CODE_ANSWER:												
							//    arp_Parser()						 
							break;					
						// 
						case ARP_OP_CODE_REQUEST:						
							arp_Answer(); //  						
							break;					
						
						default:								
							// 
						
							break;
					}
					break;

				case ETHTYPE_IPv4:
					
					switch(ip_Parser()){
						case POH_ICMP:						
							switch(icmp_Parser()){
								// 
								case CODE_REPLY:												
									break;
								// 
								case CODE_REQUEST:		
										//					uart_send_buffer(RcvBuf, 100);
									icmp_Answer();
									break;
								// 
								default:			
										
									break;								 
							}
							break;

						case POH_UDP:																																			
							break;					

						default:						
							break;
					}
					break;

				case ETHTYPE_IPv6:
					break;

				default:				
					break;
			}
		
		}	
	
}
