#ifndef __ETH_DEF_H__
#define __ETH_DEF_H__

#include "misc.h"

//------------------------------------------------------------
#define PHY_ADDR			0x1C
#define PHY_DEFAULT_MODE	PHY_MODE_10BaseT_FDn

//------------------------------------------------------------
//       BAG  JitterWnd  
#define PSC_VAL(v)	(CPU_CLK * (v) - 1)
//        
#define BAG_PERIOD(t)	(CPU_CLK * (t))
//         
#define JITTER_WND(t)	(CPU_CLK * (t) - 1)

//   
#define EXCH_ATTEMPT	10
//   
#define COLL_WND		(1ul << 7)

//------------------------------------------------------------
//------------------------------------------------------------
//    PHY
//  
#define bPHY_BN_nRST				BM(0)

#define PHY_CTRL_MODE				1
#define mPHY_CTRL_MODE				(0x7ul << PHY_CTRL_MODE)

#define bPHY_CTRL_FX_EN				BM(7)
#define bPHY_CTRL_MDI				BM(8)
#define bPHY_CTRL_MDIO_SEL			BM(9)
#define bPHY_CTRL_MDC				BM(10)

#define PHY_CTRL_PHYADD				11	
#define mPHY_CTRL_PHYADD			(0x1Ful << PHY_CTRL_PHYADD)	

//  
enum{
	PHY_MODE_10BaseT_HDn, 	// n -  
	PHY_MODE_10BaseT_FDn,
	PHY_MODE_100BaseT_HDn,
	PHY_MODE_100BaseT_FDn,
	PHY_MODE_100BaseT_HD,	
	PHY_MODE_TRANSLATOR, 	// 
	PHY_MODE_LOW_CONS,		//  
	PHY_MODE_AUTO,			//  
	PHY_MODE_CNT	
};

//------------------------------------------------------------
//   PHY
#define bPHY_STAT_LED0		BM(0)	// 0 - 100, 1 - 10
#define bPHY_STAT_LED1		BM(1)	// 0 - link , 1 - link 
#define bPHY_STAT_LED2		BM(2)	// 0 -  CRS, 1 -  CRS
#define bPHY_STAT_LED3		BM(3)	// 0 - fill-duplex, 1 - half-duplex
#define bPHY_STAT_READY		BM(4)	// 0 -  , 1 - 
#define bPHY_STAT_CRS		BM(5)	// 1 -  
#define bPHY_STAT_COL		BM(6)	// 1 - 
#define bPHY_STAT_FX_VALID	BM(8)	// 1 -    FX
#define bPHY_STAT_MDO		BM(9)	//    SMI    
#define bPHY_STAT_MDINT		BM(10)	//    PHY

//------------------------------------------------------------
//------------------------------------------------------------
//    ethernet
#define ETH_RAM_BASE_ADDR	(unsigned int)0x38000000
//    ethernet
#define ETH_RAM_FULL_SIZE	8192	//  
//  
#define ETH_RAM_HALF	(ETH_RAM_FULL_SIZE >> 1)
//   
#define ETH_RAM_SIZE_R	ETH_RAM_HALF
//   
#define ETH_RAM_SIZE_X	ETH_RAM_HALF
//      
#define ETH_RAM_BASE_R		ETH_RAM_BASE_ADDR
#define ETH_RAM_BASE_X		(ETH_RAM_BASE_ADDR + ETH_RAM_SIZE_R)

//------------------------------------------------------------
//------------------------------------------------------------
// Ethernet

//    
//    
#define mETH_X_LENGTH		0xFFFFul
//    
#define ETH_X_STAT_RCNT		16
#define mETH_X_STAT_RCNT	(0xFul << ETH_X_STAT_RCNT)
//     
#define bETH_X_STAT_RL		BM(20)
//   late collision    
#define bETH_X_STAT_LC		BM(21)
//    
#define bETH_X_STAT_UR		BM(22)

//------------------------------------------------------------
//    
//        CRC
#define mETH_R_LENGTH		0xFFFFul
//   PAUSE
#define bETH_R_PF_ERR		BM(16)
//   
#define bETH_R_CF_ERR		BM(17)
//     1518 
#define bETH_R_LF_ERR		BM(18)
//    
#define bETH_R_SF_ERR		BM(19)
//        
//    
#define bETH_R_LEN_ERR		BM(20)
//       8
#define bETH_R_DN_ERR		BM(21)
//   CRC 
#define bETH_R_CRC_ERR		BM(22)
//      nibbles
#define bETH_R_SMB_ERR		BM(23)
//   
#define bETH_R_MCA			BM(24)
//   
#define bETH_R_BCA			BM(25)
//   
#define bETH_R_UCA			BM(26)

//------------------------------------------------------------
//    
//   
#define mETH_CTRL_COL_WND	0xFFul
//    PAUSE
#define bETH_CTRL_PAUSE_EN	BM(8)
//    
#define bETH_CTRL_DTRM_EN	BM(9)
//   
#define bETH_CTRL_HD_EN		BM(10)
//      ???
#define bETH_CTRL_EXT_EN	BM(11)
//   
#define ETH_CTRL_BUFF_MODE	12
//  
enum{
	BWM_LIN,	//  
	BWM_AUTO,	//   
	BWM_FIFO	
};
#define mETH_CTRL_BUFF_MODE	(0x3ul << ETH_CTRL_BUFF_MODE)
//   ???
#define bETH_CTRL_RCLR_EN	BM(14)	//   1 - 
#define bETH_CTRL_XRST		BM(16)	//   1 - 
//  
#define bETH_CTRL_RRST		BM(17)
//  
#define bETH_CTRL_DLB		BM(18)
//     FIFO
//    
#define bETH_CTRL_DBG_RF_EN	BM(28)
//     FIFO
// 
#define bETH_CTRL_DBG_XF_EN	BM(29)
//     
#define ETH_DBG_MODE		30
enum{
	DBG_WM_FREERUN,
	DBG_WM_HALT = 2,
	DBG_WM_STOP
};
#define mETH_DBG_MODE		(0x3ul << ETH_DBG_MODE)

//------------------------------------------------------------
//   
//  -  
#define mETH_X_CFG_RTRY_CNT		0xFul
//     
#define bETH_X_CFG_IPG_EN		BM(4)
//     CRC 
#define bETH_X_CFG_CRC_EN		BM(5)
//   
#define bETH_X_CFG_PRE_EN		BM(6)
//      PAD-
#define bETH_X_CFG_PAD_EN		BM(7)
//     EVNT[1]
#define ETH_X_CFG_EVNT_MODE		8
enum{
	EVNT_MODE_XFIFO_EMPTY,		// 
	EVNT_MODE_XFIFO_NEAR_EMPTY,	//  	
	EVNT_MODE_XFIFO_HALF,		//   
	EVNT_MODE_XFIFO_NEAR_FULL,	//  	
	EVNT_MODE_XFIFO_FULL,		// 	
	EVNT_MODE_XFIFO_SND_FNSH, 	//  
	EVNT_MODE_XFIFO_READ_WORD_FROM_BUF,	//     
	EVNT_MODE_XFIFO_NEXT_ATTM	//    
};
#define mETH_X_CFG_EVNT_MODE	(0x7ul << ETH_X_CFG_EVNT_MODE)
//       
#define bETH_X_CGF_MSB1ST		BM(12) // 0 - LSB, 1 - MSB
//   
#define bETH_X_CFG_BE			BM(14)	// 0 - little endian, 1 - big endian
//   
#define bETH_X_CFG_EN			BM(15)

//------------------------------------------------------------
//   
//     MAC-,    HASH 
#define bETH_R_CFG_MCA_EN		BM(0)
//     MAC
#define bETH_R_CFG_BCA_EN		BM(1)
//    MAC	   MAC_Address
#define bETH_R_CFG_UCA_EN		BM(2)
//     MAC
#define bETH_R_CFG_AC_EN		BM(3)
//      
#define bETH_R_CFG_EF_EN		BM(4)
//    
#define bETH_R_CFG_CF_EN		BM(5)
//      
#define bETH_R_CFG_LF_EN		BM(6)
//      
#define bETH_R_CFG_SF_EN		BM(7)

//     EVNT[1]
#define ETH_R_CFG_EVNT_MODE		8
enum{
	EVNT_MODE_RFIFO_NO_EMPTY,		//  
	EVNT_MODE_RFIFO_NEAR_NO_EMPTY,	//   	
	EVNT_MODE_RFIFO_HALF,		//   
	EVNT_MODE_RFIFO_NEAR_NO_FULL,	//   	
	EVNT_MODE_RFIFO_NO_FULL,		//  	
	EVNT_MODE_RFIFO_RCV_FNSH, 	//  
	EVNT_MODE_RFIFO_WRITE_WORD_TO_BUF,	//    
	EVNT_MODE_RFIFO_KICK		//  
};
#define mETH_R_CFG_EVNT_MODE	(0x7ul << ETH_R_CFG_EVNT_MODE)
//       
#define bETH_R_CGF_MSB1ST		BM(12) // 0 - LSB, 1 - MSB
//   
#define bETH_R_CFG_BE			BM(14)	// 0 - little endian, 1 - big endian
//   
#define bETH_R_CFG_EN			BM(15)

//------------------------------------------------------------
//   IMR   IFR  
//    
#define bETH_IMR_RF_OK			BM(0)
#define bETH_IFR_RF_OK			bETH_IMR_RF_OK			
//   -    
#define bETH_IMR_MISSED_F		BM(1)
#define bETH_IFR_MISSED_F		bETH_IMR_MISSED_F
//   
#define bETH_IMR_OVF			BM(2)
#define bETH_IFR_OVF			bETH_IMR_OVF			
//       
#define bETH_IMR_SMB_ERR		BM(3)
#define bETH_IFR_SMB_ERR		bETH_IMR_SMB_ERR		
//  CRC     CRC 
#define bETH_IMR_CRC_ERR		BM(4)
#define bETH_IFR_CRC_ERR		bETH_IMR_CRC_ERR		
//   
#define bETH_IMR_CF				BM(5)
#define bETH_IFR_CF				bETH_IMR_CF
//    
#define bETH_IMR_LF				BM(6)
#define bETH_IFR_LF				bETH_IMR_LF
//    
#define bETH_IMR_SF				BM(7)
#define bETH_IFR_SF				bETH_IMR_SF
//    
#define bETH_IMR_XF_OK			BM(8)
#define bETH_IFR_XF_OK			bETH_IMR_XF_OK
//     
#define bETH_IMR_XF_ERR			BM(9)
#define bETH_IFR_XF_ERR			bETH_IMR_XF_ERR
//   
#define bETH_IMR_UNDF			BM(10)
#define bETH_IFR_UNDF			bETH_IMR_UNDF
//  late collision  
#define bETH_IMR_LC				BM(11)
#define bETH_IFR_LC				bETH_IMR_LC				
//        
#define bETH_IMR_CRS_LOST		BM(12)
#define bETH_IFR_CRS_LOST		bETH_IMR_CRS_LOST		
//   MDIO 
#define bETH_IMR_MDIO_INT		BM(14)
#define bETH_IFR_MDIO_INT		bETH_IMR_MDIO_INT
//     MDIO 
#define bETH_IMR_MII_RDY		BM(15)
#define bETH_IFR_MII_RDY		bETH_IMR_MII_RDY

//------------------------------------------------------------
//  
//   
#define	bETH_STAT_R_EMPTY		BM(0)
//    
#define	bETH_STAT_R_AEMPTY		BM(1)
//   
#define	bETH_STAT_R_HALF		BM(2)
 //    
#define	bETH_STAT_R_AFULL		BM(3)
 //    
#define	bETH_STAT_R_FULL		BM(4)

//  ,    
#define	ETH_STAT_R_CNT			5
#define	mETH_STAT_R_CNT			(0x7ul << ETH_STAT_R_CNT)

//   
#define bETH_STAT_X_EMPTY		BM(8)
//    
#define bETH_STAT_X_AEMPTY		BM(9)
//   
#define bETH_STAT_X_HALF		BM(10)
//    
#define bETH_STAT_X_AFULL		BM(11)
//   
#define bETH_STAT_X_FULL		BM(12)

//------------------------------------------------------------
//   MDIO
//   PHY
#define mETH_PHY_MDIO_REG			0x1Ful
//      MDIO
#define ETH_PHY_MDIO_DIV			5
#define mETH_PHY_MDIO_DIV			(0x7ul << ETH_PHY_MDIO_DIV)
//  
#define ETH_PHY_MDIO_ADDR			5
#define mETH_PHY_MDIO_ADDR			(0x1Ful << ETH_PHY_MDIO_ADDR)
// 
#define bETH_PHY_MDIO_OP			BM(13)	// 1 - rd, 0 - wr
//  
#define bETH_PHY_MDIO_PRE_EN		BM(14)	// 1 -   
// /   MDIO
#define bETH_PHY_MDIO_RDY			BM(15)

#endif
