#include <string.h>

#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"
#include "eth_drv.h"
#include "eth_def.h"
#include "eth_prop.h"
#include "eth_misc.h"
#include "error.h"
#include "itmr.h"
#include "mcp23s18.h"
#include "uart.h"


static TEthSettings EthSetting;
static TEthAddr EthernetMAC;
static TIP4Addr EthernetIP;
static TIP4Addr SubNetMask;
static TIP4Addr Gateway;

TXSTATUS TxStatus;
RXSTATUS ptrRxStatus;

typedef unsigned char byte;

static byte Enc28j60Bank;

static unsigned int nextpckptr = RXSTART_INIT;

#define enableChip()	PORT_WriteBit(MDR_PORTD, PORT_Pin_3, Bit_RESET)
#define disableChip() PORT_WriteBit(MDR_PORTD, PORT_Pin_3, Bit_SET)
#define xferSPI		SendAndRcvByte

#define ENC28J60_MINFRAME	64
#define ENC28J60_MAXFRAME	1518

#define CYSWAP_ENDIAN16(x) ((unsigned int)(((x) << 8) | ((x) >> 8)))
	

/*Macro for Silicon Errata to do with Transmit Logic Reset.
Silicon Errata No.12 as per Latest Errata doc for ENC28J60
See http://ww1.microchip.com/downloads/en/DeviceDoc/80349c.pdf */
#define ERRATAFIX   writeOp(ENC28J60_BIT_FIELD_SET,ECON1, ECON1_TXRST);writeOp(ENC28J60_BIT_FIELD_CLR,ECON1, ECON1_TXRST);writeOp(ENC28J60_BIT_FIELD_CLR,EIR, EIR_TXERIF | EIR_TXIF)

static byte readOp (byte op, byte address) {
		byte result;
    enableChip();
    xferSPI(op | (address & ADDR_MASK));
    result = xferSPI(0x00);
    if (address & 0x80)
        result = xferSPI(0x00);
    
    disableChip();
    return result;
}

static void writeOp (byte op, byte address, byte data) {
    enableChip();
    xferSPI(op | (address & ADDR_MASK));
    xferSPI(data);
    disableChip();
}

static void readBuf(uint16_t len, byte* data) {
    
    enableChip();
    if (len != 0) {    
        xferSPI(ENC28J60_READ_BUF_MEM);
          
        //xferSPI(0x00); 
			
        while (len--) {
            *(data++) = xferSPI(0x00);
                
        }        
			}
    disableChip(); 
}
/*static unsigned int enc28j60ReadBufferWord() {
    unsigned int result;
    readBuf(2, (byte*) &result);
    return result;
}*/
static void writeBuf(uint16_t len, const byte* data) {
    enableChip();
    if (len != 0) {
        xferSPI(ENC28J60_WRITE_BUF_MEM);           
           
        while (--len) {
            xferSPI(*(data++)); 
     	};          
    }
    disableChip();
}

static void SetBank (byte address) {
    if ((address & BANK_MASK) != Enc28j60Bank) {
        writeOp(ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_BSEL1|ECON1_BSEL0);
        Enc28j60Bank = address & BANK_MASK;
        writeOp(ENC28J60_BIT_FIELD_SET, ECON1, Enc28j60Bank>>5);
    }
}

 unsigned char readRegByte (unsigned char address) {
    SetBank(address);
    return readOp(ENC28J60_READ_CTRL_REG, address);
}

unsigned int readReg(char address) {
    return readRegByte(address) + (readRegByte(address+1) << 8);
}

static void writeRegByte (byte address, byte data) {
    SetBank(address);
    writeOp(ENC28J60_WRITE_CTRL_REG, address, data);
}

static void writeReg(byte address, uint16_t data) {
    writeRegByte(address, data & 0xFF);
    writeRegByte(address + 1, (data >> 8) & 0xFF);
}

unsigned int readPhyByte (char address) {
    writeRegByte(MIREGADR, address);
    writeRegByte(MICMD, MICMD_MIIRD);
    while (readRegByte(MISTAT) & MISTAT_BUSY)
        ;
    writeRegByte(MICMD, 0x00);
    return readReg(MIRD);
}

static void writePhy (byte address, uint16_t data) {
    writeRegByte(MIREGADR, address);
    writeReg(MIWR, data);
    while (readRegByte(MISTAT) & MISTAT_BUSY)
        ;
}

static void ethernet_Hardware_Init(void)
{
	PORT_InitTypeDef PORT_InitStructure;
	
	
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTA | RST_CLK_PCLK_PORTD, ENABLE);	
	// Ethernet CS
		
	PORT_InitStructure.PORT_PULL_UP = PORT_PULL_UP_OFF;
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;
	
	PORT_InitStructure.PORT_Pin = (PORT_Pin_3); // CS
	PORT_Init(MDR_PORTD, &PORT_InitStructure);	
	PORT_WriteBit(MDR_PORTD, PORT_Pin_3, Bit_SET);
	
	
	// Ethernet Interrput
	PORT_InitStructure.PORT_PULL_UP = PORT_PULL_UP_ON;
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	PORT_InitStructure.PORT_Pin = (PORT_Pin_0); // INT1
	PORT_Init(MDR_PORTA, &PORT_InitStructure);	
		
}

//---------------------------------------------------------------------------------------------
//    PHY  MAC 
//---------------------------------------------------------------------------------------------

unsigned char ethernet_LowLevelInit(void)
{	byte rev;

	ethernet_Hardware_Init();
	
  	writeOp(ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET);
    itmr_Delay(2); // errata B7/2
    /*while (!readOp(ENC28J60_READ_CTRL_REG, ESTAT) & ESTAT_CLKRDY)
        ;
*/
    writeReg(ERXST, RXSTART_INIT);
    writeReg(ERXRDPT, RXSTART_INIT);
    writeReg(ERXND, RXSTOP_INIT);
    writeReg(ETXST, TXSTART_INIT);
    writeReg(ETXND, TXSTOP_INIT);

   
	writeRegByte(ERXFCON, ERXFCON_UCEN|ERXFCON_CRCEN|ERXFCON_PMEN|ERXFCON_BCEN);
	
    writeReg(EPMM0, 0x303f);
    writeReg(EPMCS, 0xf7f9);		//0xf739
	
	ethprop_GetMAC(&EthernetMAC);
	EthSetting.mac[0] = EthernetMAC.addr[0];
	EthSetting.mac[1] = EthernetMAC.addr[1];
	EthSetting.mac[2] = EthernetMAC.addr[2];
	EthSetting.mac[3] = EthernetMAC.addr[3];
	EthSetting.mac[4] = EthernetMAC.addr[4];
	EthSetting.mac[5] = EthernetMAC.addr[5];
	
	ethprop_GetIP(&EthernetIP);
	EthSetting.ip[0] = EthernetIP.d8[0];
	EthSetting.ip[1] = EthernetIP.d8[1];
	EthSetting.ip[2] = EthernetIP.d8[2];
	EthSetting.ip[3] = EthernetIP.d8[3];	
	
	ethprop_GetMask(&SubNetMask);
	EthSetting.mask[0] = SubNetMask.d8[0];
	EthSetting.mask[1] = SubNetMask.d8[1];
	EthSetting.mask[2] = SubNetMask.d8[2];
	EthSetting.mask[3] = SubNetMask.d8[3];	
	
	ethprop_GetGate(&Gateway);
	EthSetting.gateway[0] = Gateway.d8[0];	
	EthSetting.gateway[1] = Gateway.d8[1];	
	EthSetting.gateway[2] = Gateway.d8[2];	
	EthSetting.gateway[3] = Gateway.d8[3];		
	
	writeRegByte(MACON1, MACON1_MARXEN|MACON1_TXPAUS|MACON1_RXPAUS);
    writeRegByte(MACON2, 0x00);
    writeOp(ENC28J60_BIT_FIELD_SET, MACON3,
            MACON3_PADCFG0|MACON3_TXCRCEN|MACON3_FRMLNEN);
    writeReg(MAIPG, 0x0C12);
    writeRegByte(MABBIPG, 0x12);
    writeReg(MAMXFL, MAX_FRAMELEN);
    writeRegByte(MAADR5, EthSetting.mac[0]);
    writeRegByte(MAADR4, EthSetting.mac[1]);
    writeRegByte(MAADR3, EthSetting.mac[2]);
    writeRegByte(MAADR2, EthSetting.mac[3]);
    writeRegByte(MAADR1, EthSetting.mac[4]);
    writeRegByte(MAADR0, EthSetting.mac[5]);
		writePhy(PHCON1, PHCON1_PDPXMD); // Force full-duplex mode
		//writePhy(PHCON1, 0x0000); 
    writePhy(PHCON2, PHCON2_HDLDIS);
		writePhy(PHLCON, 0x0576);
		
		
    SetBank(ECON1);
    writeOp(ENC28J60_BIT_FIELD_SET, EIE, EIE_INTIE|EIE_PKTIE);	//enable interrupts
    writeOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_RXEN);

		//writeReg(ECON1, ECON1_RXEN | ECON1_TXRTS);
	  writeReg(ECON1, ECON1_RXEN);
    rev = readRegByte(EREVID);
    // microchip forgot to step the number on the silcon when they
    // released the revision B7. 6 is now rev B7. We still have
    // to see what they do when they release B8. At the moment
    // there is no B8 out yet
    if (rev > 5) ++rev;
    return rev;
}

//---------------------------------------------------------------------------------------------
void ethernet_MACConfig(void)
{
//	unsigned char cnt;

	//ethernet_LowLevelInit();
		writeRegByte(MAADR5, EthSetting.mac[0]);
    writeRegByte(MAADR4, EthSetting.mac[1]);
    writeRegByte(MAADR3, EthSetting.mac[2]);
    writeRegByte(MAADR2, EthSetting.mac[3]);
    writeRegByte(MAADR1, EthSetting.mac[4]);
    writeRegByte(MAADR0, EthSetting.mac[5]);
	
/*
	itmr_Delay(3000);
	//  arp    ip  
	//   ""  		
	for(cnt = 0; cnt < ARP_START_CNT; cnt++)
		ARP_START();					
*/	
}


bool ethernet_isLinkUp() {
    return (readPhyByte(PHSTAT2) >> 2) & 1;
}

unsigned int enc28j60PacketReceive(unsigned char* packet, unsigned int maxLen)
	{
	unsigned int pckLen;
	
		
	/*//Check if Link is Up
    if(IsLinkUp()==0){
        return FALSE;
    }
	*/
    /*Read EPKTCNT to see if we have any packets in.*/
    
    if(readRegByte(EPKTCNT) == 0){
        return 0;//Report that No Proper Packets RX'd.
    }
        
    /*Setup memory pointers to Read in this RX'd packet.*/
    
    writeReg(ERDPT, nextpckptr);
    
    /*Read in the Next Packet Pointer,and the following 32bit Status Vector.
    See FIGURE 7-3: SAMPLE RECEIVE PACKET LAYOUT on Page 45 of the datasheet.*/
    readBuf(6,(unsigned char*)&ptrRxStatus.v[0]);
    
    /*Because,Little Endian.*/
   
    nextpckptr = ptrRxStatus.bits.NextPacket;
		
    /*Compute actual length of the RX'd Packet.*/
   
    pckLen = ptrRxStatus.bits.ByteCount;
		
	if( pckLen > (maxLen-1) ){
	pckLen = maxLen;
	}
	
    /*Read the packet only if it was RX'd Okay.
    We should be checking other flags too,like Length Out of Range,
    but that one doesnt seem reliable.
    We need more work and testing here.*/
    if(ptrRxStatus.bits.RxOk==0x01){
        readBuf(pckLen,packet);//Read packet into buffer.
    }

    /*Ensure that ERXRDPT is Always ODD! Else Buffer gets corrupted.
    See No.5 in the Silicon Errata*/                                      
    
    if ( ((nextpckptr - 1) < RXSTART_INIT) || ((nextpckptr-1) > RXSTOP_INIT) ) {
        /*Free up memory in that 8kb buffer by adjusting the RX Read pointer,
        since we are done with the packet.*/
        writeReg(ERXRDPT, RXSTOP_INIT);
        
    }else{
        writeReg(ERXRDPT, ( nextpckptr - 1 ) );
        
    }
		SetBank(ECON2);
  /*To signal that we are done with the packet,decrement EPKTCNT*/
  writeOp(ENC28J60_BIT_FIELD_SET, ECON2, ECON2_PKTDEC);
  
  /*Return the length of the packet RX'd*/
  return pckLen;
		
}


unsigned char enc28j60_send_packet(unsigned char *data, unsigned int len){

    unsigned char  bytControl=0x00;
  	
	/*Check if Link is Up*/
   /* if(IsLinkUp()==0){
        return FALSE;
    }*/
	
    
    
    /*Buffer write ptr to start of Tx packet*/
    writeReg(ETXST,TXSTART_INIT);        
    
	/*Set write buffer pointer to point to start of Tx Buffer*/
	writeReg(EWRPT,TXSTART_INIT);        
	
    
    /*Write the Per Packet Control Byte
    See FIGURE 7-1: FORMAT FOR PER PACKET CONTROL BYTES
    on Page 41 of the datasheet */
    writeBuf(2,&bytControl);
      
    /*Write the packet into the ENC's buffer*/
	writeBuf(len,data);  
    
	/*Tell MAC when the end of the packet is*/
	writeReg(ETXND, ( (len+TXSTART_INIT+1) ));       
	
    /*We would like to enable Interrupts on Packet TX complete.*/
    writeOp(ENC28J60_BIT_FIELD_CLR,EIR,EIR_TXIF);
    writeOp(ENC28J60_BIT_FIELD_SET,EIE, EIE_TXIE |EIE_INTIE);
    
    /*Macro for Silicon Errata to do with Transmit Logic Reset.
    Silicon Errata No.12 as per Latest Errata doc for ENC28J60
    See http://ww1.microchip.com/downloads/en/DeviceDoc/80349c.pdf */
    //ERRATAFIX;    
    
    /*Send that Packet!*/
    writeOp(ENC28J60_BIT_FIELD_SET,ECON1, ECON1_TXRTS);
    
    /*Wait for the Chip to finish the TX,and
      read the TX interrrupt bit to check the same.*/
    do{}while (!(readRegByte(EIR) & (EIR_TXIF)));             // kill some time. Note: Nice place to block?             // kill some time. Note: Nice place to block?

    /*Clear TXRTS,since the packet has been TX'd.*/
    writeOp(ENC28J60_BIT_FIELD_CLR,ECON1, ECON1_TXRTS);
  
    /*We will now attempt to read the TX Status Vectors.
    See TABLE 7-1: TRANSMIT STATUS VECTORS on Page 43 of the datasheet.*/
    //SetBank(0);
    
    /*Because,that control byte.*/
    len++;
    
    /*Configure the buffer read ptr to read status structure*/
    writeReg(ERDPT, len);       
    
    
    /*Read In the TX Status Vectors*/
    /*Note: Use these for debugging.Really useful.*/
    readBuf(7,&TxStatus.v[0]);

    /*Read TX status vectors to see if TX was interrupted.*/
    if (readRegByte(ESTAT) & ESTAT_TXABRT){
        if (TxStatus.bits.LateCollision){
            writeOp(ENC28J60_BIT_FIELD_CLR,ECON1, ECON1_TXRTS);//Toggle the TXRTS
            writeOp(ENC28J60_BIT_FIELD_SET,ECON1, ECON1_TXRTS);
            writeOp(ENC28J60_BIT_FIELD_CLR,ESTAT,ESTAT_TXABRT | ESTAT_LATECOL);//Clear the Late Collision Bit.
        }
    writeOp(ENC28J60_BIT_FIELD_CLR,EIR, EIR_TXERIF | EIR_TXIF);//Clear the Interrupt Flags.
    writeOp(ENC28J60_BIT_FIELD_CLR,ESTAT,ESTAT_TXABRT);//Clear the Abort Flag.
    return false;//Report a Failed Packet TX.
  }else{
    return true;//Packet Sent Okay! :-)
  }

}

//---------------------------------------------------------------------------------------------
void ethernet_GetSettings(TEthSettings* settings)
{
	memcpy(settings, &EthSetting, sizeof(TEthSettings));
}

void ethernet_SetSettings(TEthSettings* settings)
{
	memcpy(&EthSetting, settings, sizeof(TEthSettings));
			
	EthernetMAC.addr[0] = EthSetting.mac[0];
	EthernetMAC.addr[1] = EthSetting.mac[1];
	EthernetMAC.addr[2] = EthSetting.mac[2];
	EthernetMAC.addr[3] = EthSetting.mac[3];
	EthernetMAC.addr[4] = EthSetting.mac[4];
	EthernetMAC.addr[5] = EthSetting.mac[5];
	ethprop_SetMAC(&EthernetMAC);
	
	EthernetIP.d8[0] = EthSetting.ip[0];
	EthernetIP.d8[1] = EthSetting.ip[1];
	EthernetIP.d8[2] = EthSetting.ip[2];
	EthernetIP.d8[3] = EthSetting.ip[3];
	ethprop_SetIP(&EthernetIP);	
	
	SubNetMask.d8[0] = EthSetting.mask[0];
	SubNetMask.d8[1] = EthSetting.mask[1];
	SubNetMask.d8[2] = EthSetting.mask[2];
	SubNetMask.d8[3] = EthSetting.mask[3];
	ethprop_SetMask(&SubNetMask);	
	
	Gateway.d8[0] = EthSetting.gateway[0];
	Gateway.d8[1] = EthSetting.gateway[1];
	Gateway.d8[2] = EthSetting.gateway[2];
	Gateway.d8[3] = EthSetting.gateway[3];	
	ethprop_SetGate(&Gateway);	
	
	ethernet_MACConfig();
}
void check_regs ( void)
{		char buffer[32];
		
	  itmr_Delay(3000);
    sprintf(buffer, " %x --> %x \n\r", readReg(ERXST), RXSTART_INIT);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
    sprintf(buffer, " %x --> %x \n\r", readReg(ERXRDPT), RXSTART_INIT);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
	
		
  sprintf(buffer, " %x --> %x \n\r", readReg(ERXND), RXSTOP_INIT);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);  
	
		
    sprintf(buffer, " %x --> %x \n\r", readReg(ETXST), TXSTART_INIT);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
  
		
		sprintf(buffer, " %x --> %x \n\r", readReg(ETXND), TXSTOP_INIT);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
    
		sprintf(buffer, " %x --> %x \n\r", readRegByte(ERXFCON), ERXFCON_UCEN|ERXFCON_CRCEN|ERXFCON_PMEN|ERXFCON_BCEN);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
    
		sprintf(buffer, " %x --> %x \n\r", readReg(EPMM0), 0x303f);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
    
		sprintf(buffer, " %x --> %x \n\r", readReg(EPMCS), 0xf7f9);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
		
		sprintf(buffer, " %x --> %x \n\r", readRegByte(MACON1), MACON1_MARXEN|MACON1_TXPAUS|MACON1_RXPAUS);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
		
    
    sprintf(buffer, " %x --> %x \n\r", readRegByte(MACON2), 0x00);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
		
		sprintf(buffer, " %x --> %x \n\r", readRegByte(MACON3), MACON3_PADCFG0|MACON3_TXCRCEN|MACON3_FRMLNEN);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
		
    
		sprintf(buffer, " %x --> %x \n\r", readReg(MAIPG), 0x0C12);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
    
		sprintf(buffer, " %x --> %x \n\r", readRegByte(MABBIPG), 0x12);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
    
		sprintf(buffer, " %x --> %x \n\r", readReg(MAMXFL), MAX_FRAMELEN);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
    
		sprintf(buffer, " %x --> %x \n\r", readRegByte(MAADR5), EthSetting.mac[0]);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		sprintf(buffer, " %x --> %x \n\r", readRegByte(MAADR4), EthSetting.mac[1]);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);		
		sprintf(buffer, " %x --> %x \n\r", readRegByte(MAADR3), EthSetting.mac[2]);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);    
		sprintf(buffer, " %x --> %x \n\r", readRegByte(MAADR2), EthSetting.mac[3]);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
    sprintf(buffer, " %x --> %x \n\r", readRegByte(MAADR1), EthSetting.mac[4]);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
    sprintf(buffer, " %x --> %x \n\r", readRegByte(MAADR0), EthSetting.mac[5]);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
		
		sprintf(buffer, " %x --> %x \n\r", readPhyByte(PHCON1), PHCON1_PDPXMD);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
    sprintf(buffer, " %x --> %x \n\r", readPhyByte(PHCON2), PHCON2_HDLDIS);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
		
		
		sprintf(buffer, " %x --> %x \n\r", readReg(ECON1), ECON1_TXRST);
		uart_SendBuf(buffer,32);		
		itmr_Delay(100);
}
void test_receive_packet (void)
{
	unsigned char buf[576];
	enc28j60PacketReceive(buf,500);
	uart_send_buffer(buf,64);
}
