#include "MDR32Fx.h"
#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"

#include "misc.h"
#include "encoder.h"
#include "itmr.h"
#include "led.h"
#include "main.h"

//#define __DEBUG_ENCODER
#ifdef __DEBUG_ENCODER
	#include "led.h"
	#include "uart.h"
#endif

static unsigned int CurEncCnt = 0, OldEncCnt = 0;
#define ENC_MIN_VALUE		1
#define ENC_MAX_VALUE		19
static volatile bool FirstLevelChange = false;
static volatile unsigned int TimerWait;

void encoder_Init(void)
{
	PORT_InitTypeDef PORT_InitStructure;
	//  
	RST_CLK_PCLKcmd(RST_CLK_PCLK_RST_CLK | RST_CLK_PCLK_PORTA, ENABLE);

	PORT_InitStructure.PORT_OE = PORT_OE_IN;	
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_PULL_UP = PORT_PULL_UP_OFF;
	PORT_InitStructure.PORT_PULL_DOWN = PORT_PULL_DOWN_OFF;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	
	PORT_InitStructure.PORT_Pin = PORT_Pin_3;	
	PORT_Init(MDR_PORTA, &PORT_InitStructure);	
	
	timcap1_Init();			
	CurEncCnt = OldEncCnt = ENC_MIN_VALUE;
}

//      ENC_A
void encoder_IRQProc(void)
{				
	timcap1_DeInit();
	FirstLevelChange = true;
	TimerWait	= itmr_GetTick();	
	if(PORT_ReadInputData(MDR_PORTA) & PORT_Pin_3){	//   
		if(CurEncCnt < ENC_MAX_VALUE){
			CurEncCnt++;										
		}
	
	}
	else{				//   
		if(CurEncCnt > ENC_MIN_VALUE){
			CurEncCnt--;							
		}
	
	}		
	
}

bool encoder_GetValue(unsigned int* enc)
{	
	static unsigned char state = 0;	
	bool res = false;
	switch(state){
		case 0:	//   		
			if(FirstLevelChange) state = 1;						
			break;
		case 1:	//  
			if(itmr_IsTime(TimerWait, 20 /*50*/)){										
				if(OldEncCnt != CurEncCnt){				
					OldEncCnt = CurEncCnt;
					*enc = CurEncCnt;
					TimerWait	= itmr_GetTick();	
					res = true;
				}	
				
				state = 2;
			}				
			break;		
		case 2:
			if(itmr_IsTime(TimerWait, 100/*200*/)){
				state = 0;
				FirstLevelChange = false;					
				timcap1_Init();
			}
			break;
	}	
	return res;
}

unsigned int encoder_GetMax(void)
{
	return ENC_MAX_VALUE;
}

void encoder_Test(void)
{
	unsigned int enc;
	if(encoder_GetValue(&enc)){
		led_Toggle(1);
	}
}
