#include "MDR32F9Qx_port.h"
#include "MDR32F9Qx_rst_clk.h"
#include "MDR32F9Qx_ebc.h"

#include "itmr.h"

#define READ_1st	0x00
#define READ_2nd	0x30

#define READ_FOR_COPY_BACK_1st	0x00
#define READ_FOR_COPY_BACK_2nd	0x35

#define READ_ID	0x90

#define RESET	0xFF

#define PAGE_PROGRAM_1st	0x80
#define PAGE_PROGRAM_2nd	0x10

#define COPY_BACK_PROGRAM_1st	0x85
#define COPY_BACK_PROGRAM_2nd	0x10

#define BLOCK_ERASE_1st	0x60
#define BLOCK_ERASE_2nd	0xD0

#define RANDOM_DATA_IN	0x85

#define RANDOM_DATA_OUT_1st	0x05
#define RANDOM_DATA_OUT_2nd	0xE0

#define READ_STATUS	0x70


static EBC_InitTypeDef EBC_InitStruct;
//static EBC_MemRegionInitTypeDef EBC_MemRegionInitStruct;

static unsigned int nand_cmd_phase_without_stop(unsigned char start_cmd, unsigned int cycle_cnt)
{
	return (0x77 << 24) | ((cycle_cnt & 0x7) << 21) | (start_cmd << 3);
}
/*
static unsigned int nand_cmd_phase_with_stop(unsigned char start_cmd, unsigned char stop_cmd, unsigned int cycle_cnt)
{
	return (0x77 << 24) | ((cycle_cnt & 0x7) << 21) | (1 << 20) | (stop_cmd << 11) | (start_cmd << 3);
}
*/
static unsigned int nand_data_phase(unsigned char stop_cmd)
{		
	return (0x77 << 24) | (1 << 19) | (stop_cmd << 11);
}

//------------------------------------------------------------
void nand_flash_Init(void)
{
	PORT_InitTypeDef PORT_InitStructure;
	
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTA | RST_CLK_PCLK_PORTC | RST_CLK_PCLK_PORTD | RST_CLK_PCLK_PORTE | RST_CLK_PCLK_PORTF, ENABLE);			
	// portA - data/cmd bus	
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_MAIN;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	
	PORT_InitStructure.PORT_Pin = PORT_Pin_0 | PORT_Pin_1 | PORT_Pin_2 | PORT_Pin_3 |  
																PORT_Pin_4 | PORT_Pin_5 | PORT_Pin_6 | PORT_Pin_7;	
	PORT_Init(MDR_PORTA, &PORT_InitStructure);	
	
	// 0 - NWR, 1 - NRD, 2 - CLKO
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_MAIN;
	PORT_InitStructure.PORT_MODE = PORT_MODE_DIGITAL;
	PORT_InitStructure.PORT_SPEED = PORT_SPEED_MAXFAST;
	
	PORT_InitStructure.PORT_Pin = PORT_Pin_0 | PORT_Pin_1 | PORT_Pin_2;	
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	
	
	// BUSY
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	PORT_InitStructure.PORT_Pin = PORT_Pin_15;	
	PORT_Init(MDR_PORTC, &PORT_InitStructure);	
	
	// cmd lanch
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_ALTER;
	PORT_InitStructure.PORT_Pin = PORT_Pin_1;	
	PORT_Init(MDR_PORTD, &PORT_InitStructure);	
	
	// CS
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_Pin = PORT_Pin_0;	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
	PORT_Init(MDR_PORTE, &PORT_InitStructure);	
	PORT_WriteBit(MDR_PORTE, PORT_Pin_0, Bit_RESET);
	
	// write protect
	PORT_InitStructure.PORT_FUNC = PORT_FUNC_PORT;
	PORT_InitStructure.PORT_Pin = PORT_Pin_2;	
	PORT_InitStructure.PORT_OE = PORT_OE_OUT;	
	PORT_Init(MDR_PORTF, &PORT_InitStructure);	
	PORT_WriteBit(MDR_PORTF, PORT_Pin_2, Bit_SET);
	
	// ext bus config
	RST_CLK_PCLKcmd(RST_CLK_PCLK_EBC, ENABLE);
	EBC_StructInit(&EBC_InitStruct);
	EBC_InitStruct.EBC_Mode					 = EBC_MODE_NAND;
  EBC_InitStruct.EBC_NandTrc       = EBC_NAND_CYCLES_6HCLK;
  EBC_InitStruct.EBC_NandTwc       = EBC_NAND_CYCLES_6HCLK;
  EBC_InitStruct.EBC_NandTrea      = EBC_NAND_CYCLES_4HCLK;
  EBC_InitStruct.EBC_NandTwp       = EBC_NAND_CYCLES_3HCLK;
  EBC_InitStruct.EBC_NandTwhr      = EBC_NAND_CYCLES_6HCLK;
  EBC_InitStruct.EBC_NandTalea		 = EBC_NAND_CYCLES_10HCLK;
  EBC_InitStruct.EBC_NandTrr       = EBC_NAND_CYCLES_2HCLK;	
	//EBC_InitStruct.EBC_UseMemRegion  = ENABLE;
	EBC_Init(&EBC_InitStruct);	
/*
	EBC_MemRegionStructInit(&EBC_MemRegionInitStruct);
	EBC_MemRegionInit(&EBC_MemRegionInitStruct, EBC_MEM_REGION_70000000);	
*/	
}

unsigned int nand_flash_ReadID(void)
{
	char id[5];
	unsigned int addr = nand_cmd_phase_without_stop(READ_ID, 1);	
	unsigned int addr_data = nand_data_phase(0);
 	*(volatile unsigned char*)addr = 0x00;
	
	
	id[0] = *(volatile unsigned char*)addr_data;
	id[1] = *(volatile unsigned char*)addr_data;
	id[2] = *(volatile unsigned char*)addr_data;
	id[3] = *(volatile unsigned char*)addr_data;
	id[4] = *(volatile unsigned char*)addr_data;
	
	
	return (id[0] << 16) | (id[1] << 8) | id[2];
}

